@@ -81,9 +81,9 @@ pub enum ValidationErrorKind {
     /// The input value doesn't match any of specified options.
     Enum { options: Value },
     /// Value is too large.
-    ExclusiveMaximum { limit: f64 },
+    ExclusiveMaximum { limit: Value },
     /// Value is too small.
-    ExclusiveMinimum { limit: f64 },
+    ExclusiveMinimum { limit: Value },
     /// Everything is invalid for `false` schema.
     FalseSchema,
     /// If the referenced file is not found during ref resolution.
@@ -103,7 +103,7 @@ pub enum ValidationErrorKind {
     /// Too many items in an array.
     MaxItems { limit: u64 },
     /// Value is too large.
-    Maximum { limit: f64 },
+    Maximum { limit: Value },
     /// String is too long.
     MaxLength { limit: u64 },
     /// Too many properties in an object.
@@ -111,7 +111,7 @@ pub enum ValidationErrorKind {
     /// Too few items in an array.
     MinItems { limit: u64 },
     /// Value is too small.
-    Minimum { limit: f64 },
+    Minimum { limit: Value },
     /// String is too short.
     MinLength { limit: u64 },
     /// Not enough properties in an object.
@@ -364,7 +364,7 @@ impl<'a> ValidationError<'a> {
         schema_path: JSONPointer,
         instance_path: JSONPointer,
         instance: &'a Value,
-        limit: f64,
+        limit: Value,
     ) -> ValidationError<'a> {
         ValidationError {
             instance_path,
@@ -377,7 +377,7 @@ impl<'a> ValidationError<'a> {
         schema_path: JSONPointer,
         instance_path: JSONPointer,
         instance: &'a Value,
-        limit: f64,
+        limit: Value,
     ) -> ValidationError<'a> {
         ValidationError {
             instance_path,
@@ -468,7 +468,7 @@ impl<'a> ValidationError<'a> {
         schema_path: JSONPointer,
         instance_path: JSONPointer,
         instance: &'a Value,
-        limit: f64,
+        limit: Value,
     ) -> ValidationError<'a> {
         ValidationError {
             instance_path,
@@ -520,7 +520,7 @@ impl<'a> ValidationError<'a> {
         schema_path: JSONPointer,
         instance_path: JSONPointer,
         instance: &'a Value,
-        limit: f64,
+        limit: Value,
     ) -> ValidationError<'a> {
         ValidationError {
             instance_path,
@@ -656,15 +656,6 @@ impl<'a> ValidationError<'a> {
         }
     }
 
-    pub(crate) fn schema(instance: &'a Value) -> ValidationError<'a> {
-        ValidationError {
-            instance_path: JSONPointer::default(),
-            instance: Cow::Borrowed(instance),
-            kind: ValidationErrorKind::Schema,
-            schema_path: JSONPointer::default(),
-        }
-    }
-
     pub(crate) fn null_schema() -> ValidationError<'a> {
         ValidationError {
             instance_path: JSONPointer::default(),
@@ -779,13 +770,13 @@ impl fmt::Display for ValidationError<'_> {
     #[inline]
     fn fmt(&self, f: &mut Formatter<'_>) -> fmt::Result {
         match &self.kind {
-            ValidationErrorKind::Schema => write!(f, "Schema error"),
-            ValidationErrorKind::JSONParse { error } => write!(f, "{}", error),
+            ValidationErrorKind::Schema => f.write_str("Schema error"),
+            ValidationErrorKind::JSONParse { error } => error.fmt(f),
             #[cfg(any(feature = "reqwest", test))]
-            ValidationErrorKind::Reqwest { error } => write!(f, "{}", error),
-            ValidationErrorKind::FileNotFound { error } => write!(f, "{}", error),
-            ValidationErrorKind::InvalidURL { error } => write!(f, "{}", error),
-            ValidationErrorKind::BacktrackLimitExceeded { error } => write!(f, "{}", error),
+            ValidationErrorKind::Reqwest { error } => error.fmt(f),
+            ValidationErrorKind::FileNotFound { error } => error.fmt(f),
+            ValidationErrorKind::InvalidURL { error } => error.fmt(f),
+            ValidationErrorKind::BacktrackLimitExceeded { error } => error.fmt(f),
             ValidationErrorKind::UnknownReferenceScheme { scheme } => {
                 write!(f, "Unknown scheme: {}", scheme)
             }
@@ -866,8 +857,8 @@ impl fmt::Display for ValidationError<'_> {
                     self.instance, content_media_type
                 )
             }
-            ValidationErrorKind::FromUtf8 { error } => write!(f, "{}", error),
-            ValidationErrorKind::Utf8 { error } => write!(f, "{}", error),
+            ValidationErrorKind::FromUtf8 { error } => error.fmt(f),
+            ValidationErrorKind::Utf8 { error } => error.fmt(f),
             ValidationErrorKind::Enum { options } => {
                 write!(f, "{} is not one of {}", self.instance, options)
             }
@@ -948,9 +939,7 @@ impl fmt::Display for ValidationError<'_> {
             ValidationErrorKind::Pattern { pattern } => {
                 write!(f, r#"{} does not match "{}""#, self.instance, pattern)
             }
-            ValidationErrorKind::PropertyNames { error } => {
-                write!(f, "{}", error.to_string())
-            }
+            ValidationErrorKind::PropertyNames { error } => error.fmt(f),
             ValidationErrorKind::Required { property } => {
                 write!(f, "{} is a required property", property)
             }
