@@ -13,7 +13,7 @@
 //! A schema can be compiled with two main flavours:
 //!  * using default configurations
 //! ```rust
-//! # use jsonschema::{Draft, JSONSchema};
+//! # use jsonschema::JSONSchema;
 //! # use serde_json::json;
 //! # fn foo() {
 //! # let schema = json!({"maxLength": 5});
@@ -137,6 +137,11 @@ pub(crate) mod tests_util {
             "{} should not be valid (via validate)",
             instance
         );
+        assert!(
+            !compiled.apply(instance).basic().is_valid(),
+            "{} should not be valid (via apply)",
+            instance
+        );
     }
 
     pub(crate) fn expect_errors(schema: &Value, instance: &Value, errors: &[&str]) {
@@ -164,6 +169,11 @@ pub(crate) mod tests_util {
             "{} should be valid (via validate)",
             instance
         );
+        assert!(
+            compiled.apply(instance).basic().is_valid(),
+            "{} should be valid (via apply)",
+            instance
+        );
     }
 
     pub(crate) fn validate(schema: &Value, instance: &Value) -> ValidationError<'static> {
