@@ -1,13 +1,14 @@
 //! Reference resolver. Implements logic, required by `$ref` keyword.
 //! Is able to load documents from remote locations via HTTP(S).
 use crate::{
-    compilation::{DEFAULT_ROOT_URL, DEFAULT_SCOPE},
+    compilation::DEFAULT_ROOT_URL,
     error::ValidationError,
     schemas::{id_of, Draft},
 };
 use ahash::AHashMap;
 use parking_lot::RwLock;
 use serde_json::Value;
+use std::borrow::Cow;
 use std::sync::Arc;
 use url::Url;
 
@@ -96,19 +97,16 @@ impl Resolver {
 
         // Location-independent identifiers are searched before trying to resolve by
         // fragment-less url
-        if let Some(x) = find_schemas(draft, &self.root_schema, &DEFAULT_SCOPE, &mut |id, x| {
-            if id == url.as_str() {
-                Some(x)
-            } else {
-                None
-            }
-        })? {
-            return Ok((resource, Arc::new(x.clone())));
+        if let Some(document) = self.schemas.get(url.as_str()) {
+            return Ok((resource, Arc::clone(document)));
         }
 
         // Each resolved document may be in a changed subfolder
         // They are tracked when JSON pointer is resolved and added to the resource
         let document = self.resolve_url(&resource)?;
+        if fragment.is_empty() {
+            return Ok((resource, Arc::clone(&document)));
+        }
         match pointer(draft, &document, fragment.as_ref()) {
             Some((folders, resolved)) => {
                 let joined_folders = join_folders(resource, &folders)?;
@@ -184,22 +182,44 @@ where
     Ok(None)
 }
 
+/// Searching twice is better than unconditionally allocating a String twice
+trait MaybeReplaceExt<'a> {
+    fn maybe_replace(self, needle: &str, replacement: &str) -> Cow<'a, str>;
+}
+
+impl<'a> MaybeReplaceExt<'a> for &'a str {
+    fn maybe_replace(self, needle: &str, replacement: &str) -> Cow<'a, str> {
+        if memchr::memmem::find(self.as_bytes(), needle.as_bytes()).is_some() {
+            self.replace(needle, replacement).into()
+        } else {
+            self.into()
+        }
+    }
+}
+
+impl<'a> MaybeReplaceExt<'a> for Cow<'a, str> {
+    fn maybe_replace(self, needle: &str, replacement: &str) -> Cow<'a, str> {
+        if memchr::memmem::find(self.as_bytes(), needle.as_bytes()).is_some() {
+            self.replace(needle, replacement).into()
+        } else {
+            self
+        }
+    }
+}
+
 /// Based on `serde_json`, but tracks folders in the traversed documents.
 pub(crate) fn pointer<'a, 'b>(
     draft: Draft,
     document: &'a Value,
     pointer: &'b str,
 ) -> Option<(Vec<&'a str>, &'a Value)> {
-    if pointer.is_empty() {
-        return Some((vec![], document));
-    }
     if !pointer.starts_with('/') {
         return None;
     }
     let tokens = pointer
         .split('/')
         .skip(1)
-        .map(|x| x.replace("~1", "/").replace("~0", "~"));
+        .map(|x| x.maybe_replace("~1", "/").maybe_replace("~0", "~"));
     let mut target = document;
     let mut folders = vec![];
 
@@ -209,7 +229,7 @@ pub(crate) fn pointer<'a, 'b>(
                 if let Some(id) = id_of(draft, target) {
                     folders.push(id);
                 }
-                map.get(&token)
+                map.get(&*token)
             }
             Value::Array(ref list) => parse_index(&token).and_then(|x| list.get(x)),
             _ => return None,
@@ -234,7 +254,6 @@ fn parse_index(s: &str) -> Option<usize> {
 #[cfg(test)]
 mod tests {
     use super::*;
-    use crate::JSONSchema;
     use serde_json::json;
     use std::sync::Arc;
     use url::Url;
@@ -494,27 +513,4 @@ mod tests {
         assert_eq!(resource, Url::parse("json-schema:///").unwrap());
         assert_eq!(resolved.as_ref(), schema.pointer("/definitions/a").unwrap());
     }
-
-    #[test]
-    fn id_value_is_cleaned() {
-        let schema = json!({
-            "$id": "http://foo.com/schema.json#",
-            "properties": {
-                "foo": {
-                    "$ref": "#/definitions/Bar"
-                }
-            },
-            "definitions": {
-                "Bar": {
-                    "const": 42
-                }
-            }
-        });
-        let compiled = JSONSchema::compile(&schema).unwrap();
-        // `#` should be removed
-        assert!(compiled
-            .resolver
-            .schemas
-            .contains_key("http://foo.com/schema.json"));
-    }
 }
